@@ -64,6 +64,8 @@ struct _TotemGriloPrivate {
 	Totem *totem;
 	GtkWindow *main_window;
 
+	gboolean plugins_loaded;
+
 	GrlSource *local_metadata_src;
 	GrlSource *metadata_store_src;
 	GrlSource *bookmarks_src;
@@ -145,6 +147,7 @@ typedef struct {
 
 typedef struct {
 	gboolean found;
+	GrlKeyID key;
 	GtkTreeIter *iter;
 	GrlMedia *media;
 } FindMediaData;
@@ -338,7 +341,7 @@ can_remove (GrlSource *source,
 		return CAN_REMOVE_FALSE;
 
 	scheme = g_uri_parse_scheme (url);
-	ret = g_str_equal (scheme, "file") ? CAN_REMOVE_TRUE : CAN_REMOVE_FALSE;
+	ret = (g_strcmp0 (scheme, "file") == 0) ? CAN_REMOVE_TRUE : CAN_REMOVE_FALSE;
 	g_free (scheme);
 
 	if (ret == CAN_REMOVE_TRUE)
@@ -1096,6 +1099,7 @@ find_media_cb (GtkTreeModel  *model,
 	       FindMediaData *data)
 {
 	GrlMedia *media;
+	const char *a, *b;
 
 	gtk_tree_model_get (model, iter,
 			    MODEL_RESULTS_CONTENT, &media,
@@ -1103,7 +1107,10 @@ find_media_cb (GtkTreeModel  *model,
 	if (!media)
 		return FALSE;
 
-	if (g_strcmp0 (grl_media_get_id (media), grl_media_get_id (data->media)) == 0) {
+	a = grl_data_get_string (GRL_DATA (media), data->key);
+	b = grl_data_get_string (GRL_DATA (data->media), data->key);
+
+	if (g_strcmp0 (a, b) == 0) {
 		g_object_unref (media);
 		data->found = TRUE;
 		data->iter = gtk_tree_iter_copy (iter);
@@ -1121,6 +1128,7 @@ find_media (GtkTreeModel  *model,
 	FindMediaData data;
 
 	data.found = FALSE;
+	data.key = GRL_METADATA_KEY_ID;
 	data.media = media;
 	data.iter = NULL;
 	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc) find_media_cb, &data);
@@ -2068,7 +2076,8 @@ static void
 delete_foreach (gpointer data,
 		gpointer user_data)
 {
-	GtkTreePath *path = data;
+	GtkTreeRowReference *ref = data;
+	GtkTreePath *path;
 	GtkTreeModel *view_model = user_data;
 	GtkTreeIter iter;
 	GrlSource *source;
@@ -2079,8 +2088,11 @@ delete_foreach (gpointer data,
 	GtkTreeModel *model;
 	GtkTreeIter real_model_iter;
 
-	if (!gtk_tree_model_get_iter (view_model, &iter, path))
+	path = gtk_tree_row_reference_get_path (ref);
+	if (!path || !gtk_tree_model_get_iter (view_model, &iter, path)) {
+		g_warning ("An item that was scheduled for removal isn't available any more");
 		return;
+	}
 
 	gtk_tree_model_get (view_model, &iter,
 			    MODEL_RESULTS_CONTENT, &media,
@@ -2145,14 +2157,23 @@ delete_cb (TotemSelectionToolbar *bar,
 	   TotemGrilo            *self)
 {
 	GtkTreeModel *model;
-	GList *list;
+	GList *list, *l;
 
 	g_signal_handlers_block_by_func (self->priv->browser, "view-selection-changed", self);
 
 	model = gd_main_view_get_model (GD_MAIN_VIEW (self->priv->browser));
 	list = gd_main_view_get_selection (GD_MAIN_VIEW (self->priv->browser));
+
+	/* GList of GtkTreePaths to a GList of GtkTreeRowReferences */
+	for (l = list; l != NULL; l = l->next) {
+		GtkTreeRowReference *ref;
+
+		ref = gtk_tree_row_reference_new (model, l->data);
+		gtk_tree_path_free (l->data);
+		l->data = ref;
+	}
 	g_list_foreach (list, delete_foreach, model);
-	g_list_free_full (list, (GDestroyNotify) gtk_tree_path_free);
+	g_list_free_full (list, (GDestroyNotify) gtk_tree_row_reference_free);
 
 	g_signal_handlers_unblock_by_func (self->priv->browser, "view-selection-changed", self);
 
@@ -2468,6 +2489,18 @@ totem_grilo_finalize (GObject *object)
 	G_OBJECT_CLASS (totem_grilo_parent_class)->finalize (object);
 }
 
+void
+totem_grilo_start (TotemGrilo *self)
+{
+	if (self->priv->plugins_loaded)
+		return;
+
+	g_debug ("TotemGrilo: Loading plugins");
+
+	load_grilo_plugins (self);
+	self->priv->plugins_loaded = TRUE;
+}
+
 static void
 totem_grilo_constructed (GObject *object)
 {
@@ -2478,7 +2511,6 @@ totem_grilo_constructed (GObject *object)
 	setup_ui (self);
 	grl_init (0, NULL);
 	setup_config (self);
-	load_grilo_plugins (self);
 }
 
 gboolean
@@ -2527,6 +2559,7 @@ totem_grilo_add_item_to_recent (TotemGrilo *self,
 {
 	GrlMedia *media;
 	GFile *file;
+	FindMediaData data;
 
 	g_return_val_if_fail (TOTEM_IS_GRILO (self), FALSE);
 
@@ -2568,6 +2601,19 @@ totem_grilo_add_item_to_recent (TotemGrilo *self,
 	if (!media)
 		return FALSE;
 
+	data.found = FALSE;
+	data.key = GRL_METADATA_KEY_URL;
+	data.media = media;
+	data.iter = NULL;
+	gtk_tree_model_foreach (self->priv->recent_model, (GtkTreeModelForeachFunc) find_media_cb, &data);
+
+	if (data.found) {
+		g_debug ("URI '%s' is already present in the bookmarks, not adding duplicate", uri);
+		gtk_tree_iter_free (data.iter);
+		g_object_unref (media);
+		return FALSE;
+	}
+
 	/* This should be quick, just adding the item to the DB */
 	grl_source_store_sync (self->priv->bookmarks_src,
 			       NULL,
